<?php
/**
 * Scraping Data from the Internet
 * A CS50 Seminar By Keito Uchiyama
 * November 2009
 */

/**
 * Excerpt from MyPlate Menu Library
 * Copyright (C) 2007-2009 Keito Uchiyama
 * Released under the MIT License
*/

$divider  = '<td style="border-right:1px solid #ccc;border-top:1px solid #ccc;" valign="top">';
$divider2 = '<td style="border-top:1px solid #ccc;" valign="top">';
$ignoreCategories = array(
  'SALAD BAR',
  'SANDWICH BAR',
  'FROM THE GRILLE',
  'FRUIT, FRESH, CANNED & FROZEN',
  'BROWN RICE STATION',
);
$categoryWeights = array(
  'LUNCH & DINNER ENTREES' => 0,
  'BREAKFAST MEATS' => 3,
  'BREAKFAST ENTREES' => 5,
  'ACCOMPANIMENTS' => 10,
  'SPECIAL BARS - BOARD MENU' => 15,
  'VEGETABLES' => 20,
  'BEAN, WHOLE GRAIN' => 21,
  'STARCH & POTATOES' => 60,
  'SOUPS' => 65,
  'PASTA ALA CARTE' => 70,
  'PRODUCTION SALADS' => 75,
  'BREAD,ROLLS, MISC BAKERY' => 80,
  'BREAKFAST BAKERY' => 81,
  'MAKE OR BUILD YOUR OWN' => 90,
  'DESSERTS' => 100,
);

define('HUDS_FULL', '38');   //Full menu listings; doesn't always work
define('HUDS_SIMPLE', '05'); //"Hot Entrees, Starches, Bean/Grain and Vegetables"

/* PRIVATE: Parses the menu for a particular meal
 * Returns an array with the items in the menu
 */
function parseSubMenu($rawdata) {
  global $ignoreCategories;

  //Match all categories
  preg_match_all('/<b>([^<]*)<\/b>\s*<ul>(.*?)<\/ul>/s', $rawdata, $matches);
  array_shift($matches);

  $menu = array();

  //Match every category value
  foreach ($matches[1] as $catc => $catv) {
    //If category is ignored, ignore
    if (in_array($matches[0][$catc], $ignoreCategories))
      continue;

    preg_match_all('/<li>(.*?)\s*<\/li>/', $catv, $matchen);
    array_shift($matchen);
    $menu[$matches[0][$catc]] = $matchen[0];
  }

  return $menu;
}

/* Parses the raw HTML code of a HUDS menu into arrays for each meal
 * Return value is void
 * TODO: Implement error-checking (what if there aren't three meals)
 * I think that's pretty much solved, although when there really aren't three meals,
 * this will give incorrect results...
 */
function parseHUDSMenu($rawdata, &$breakfast, &$lunch, &$dinner, $isSunday = FALSE) {
  global $divider, $divider2;

  //Remove images
  $rawdata = preg_replace('/<img src="LegendImages\/.*?\.gif" alt="" align="bottom">/', '', $rawdata);

  //Separate menu into first two blocks
  $explosion    = explode($divider2, $rawdata);
  $rawdinner    = @$explosion[1];
  $explosion2   = explode($divider, $explosion[0]);
  $rawbreakfast = @$explosion2[1];
  $rawlunch     = @$explosion2[2]; //If it's Sunday, we won't have a Lunch section

  if ((!$isSunday) && empty($rawlunch)) {
    //If it's not Sunday but lunch is empty...
    //This can happen, for unknown reasons... 2007-11-29
    //Assume that lunch is the same as dinner
    $rawlunch = $rawdinner;
  }

  //Return false is breakfast/brunch or dinner is empty
  if (empty($rawbreakfast) || empty($rawdinner)) {
    return false;
  }

  $breakfast = parseSubMenu($rawbreakfast);
  $lunch     = parseSubMenu($rawlunch);
  $dinner    = parseSubMenu($rawdinner);

  return true;
}

/* Fetches the menu for a specified date from the HUDS server
 * Returns the raw HTML code for the menu; FALSE on failure
 */
function getHUDSMenu($year, $month, $day, $type = HUDS_FULL) {
  $loc = 'http://www.foodpro.huds.harvard.edu/foodpro/center_frame.asp?myaction=read'
  .'&sName=HARVARD+UNIVERSITY+DINING+SERVICES&locationNum='.$type.'&dtdate='
  ."${month}%2F${day}%2F${year}";

  //Fetch data from HUDS server
  $ch = curl_init();
  curl_setopt($ch, CURLOPT_URL, $loc);
  curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
  //Execute; curl_exec may return FALSE on failure
  $rawmenu = curl_exec($ch);
  curl_close($ch);

  //Is HUDS telling us that no data is available?
  if (strpos($rawmenu, '<b>NO DATA AVAILABLE</b>') !== FALSE) {
    if ($type == HUDS_FULL) {
      //Try simple mode
      return getHUDSMenu($year, $month, $day, HUDS_SIMPLE);
    } else {
      //We're already in simple mode; give up
      return FALSE;
    }
  }

  return $rawmenu;
}

//////////////////////////

$rawdata = getHUDSMenu(2009,11,13);

if (parseHUDSMenu($rawdata, $bfast, $lunch, $din)) {
  echo "<pre>Breakfast:\n\n";
  var_dump($bfast);
  echo "Lunch:\n\n";
  var_dump($lunch);
  echo "Dinner:\n\n";
  var_dump($din);
}
